﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CMA.MovieRentalStore.EnterpriseModel;

namespace CMA.MovieRentalStore.EnterpriseModel.Repositories
{
    internal sealed class PersonRepository : IRepository<Person>
    {
        #region Methods
        public void Add(Person entity)
        {
            //Prüfen, ob Person bereits existiert. Wenn ja, dann nicht hinzufügen
            var existingPerson = RentalService.Current.DatabaseContext.Person.FirstOrDefault(person => person.ID == entity.ID);

            if (existingPerson == null)
            {
                RentalService.Current.DatabaseContext.AddToPerson(entity);
                RentalService.Current.DatabaseContext.SaveChanges();
            }
        }

        public void Update(Person entity)
        {
            //Prüfen, ob Person bereits existiert, Wenn nein, dann nicht aktualisieren
            var existingPerson = RentalService.Current.DatabaseContext.Person.FirstOrDefault(person => person.ID == entity.ID);

            if (existingPerson != null)
            {
                existingPerson.FirstName = entity.FirstName;
                existingPerson.LastName = entity.LastName;
                existingPerson.Age = entity.Age;
                existingPerson.Address = entity.Address;
                RentalService.Current.DatabaseContext.SaveChanges();
            }
        }

        public void Delete(int id)
        {
            //Prüfen, ob Person bereits existiert, Wenn nein, dann nicht löschen
            var existingPerson = RentalService.Current.DatabaseContext.Person.FirstOrDefault(person => person.ID == id);

            if (existingPerson != null)
            {
                RentalService.Current.DatabaseContext.Person.DeleteObject(existingPerson);
                RentalService.Current.DatabaseContext.SaveChanges();
            }
        }

        public IEnumerable<Person> GetAll(Predicate<Person> condition = null)
        {
            List<Person> foundPersons = new List<Person>();

            if (condition == null)
                foundPersons = RentalService.Current.DatabaseContext.Person.Include("Rentals").ToList();
            else
                foreach (var person in RentalService.Current.DatabaseContext.Person.Include("Rentals"))
                    if (condition(person))
                        foundPersons.Add(person);

            return foundPersons;
        }

        public Person GetEntity(int id)
        {
            var foundPerson = RentalService.Current.DatabaseContext.Person.Include("Rentals").FirstOrDefault(person => person.ID == id);
            return foundPerson;
        }
        #endregion
    }
}
