﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CMA.MovieRentalStore.EnterpriseModel.Repositories
{
    internal sealed class RentalRepository : IRepository<Rental>
    {
        #region Methods
        public void Add(Rental entity)
        {
            var existingRental = RentalService.Current.DatabaseContext.Rental.FirstOrDefault(rental => rental.ID == entity.ID);

            if (existingRental == null)
            {
                var associatedPerson = RentalService.Current.DatabaseContext.Person.FirstOrDefault(person => person.ID == entity.Person.ID);

                if (associatedPerson != null)
                {
                    associatedPerson.Rentals.Add(entity);
                    RentalService.Current.DatabaseContext.SaveChanges();
                }
            }
        }

        public void Update(Rental entity)
        {
            var existingRental = RentalService.Current.DatabaseContext.Rental.FirstOrDefault(rental => rental.ID == entity.ID);

            if (existingRental != null)
            {
                existingRental.RentalStartDate = entity.RentalStartDate;
                existingRental.RentalEndDate = entity.RentalEndDate;
                RentalService.Current.DatabaseContext.SaveChanges();
            }
        }

        public void Delete(int id)
        {
            var foundRental = RentalService.Current.DatabaseContext.Rental.Include("Person").FirstOrDefault(rental => rental.ID == id);

            if (foundRental != null)
            {
                var associatedPerson = RentalService.Current.DatabaseContext.Person.FirstOrDefault(person => person.ID == foundRental.Person.ID);

                if (associatedPerson != null)
                {
                    RentalService.Current.DatabaseContext.Rental.DeleteObject(foundRental);
                    RentalService.Current.DatabaseContext.SaveChanges();
                }
            }
        }

        public IEnumerable<Rental> GetAll(Predicate<Rental> condition = null)
        {
            var rentals = new List<Rental>();

            if (condition == null)
                rentals = RentalService.Current.DatabaseContext.Rental.Include("DVDs").Include("Person").ToList();
            else
                foreach (var rental in RentalService.Current.DatabaseContext.Rental.Include("DVDs").Include("Person"))
                    if (condition(rental))
                        rentals.Add(rental);

            return rentals;
        }

        public Rental GetEntity(int id)
        {
            var foundRental = RentalService.Current.DatabaseContext.Rental.Include("DVDs").Include("Person").FirstOrDefault(rental => rental.ID == id);
            return foundRental;
        }
        #endregion
    }
}
