﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using CMA.MovieRentalStore.EnterpriseModel;
using CMA.MovieRentalStore.EnterpriseModel.Logging;

namespace CMA.MovieRentalStore.Presentation.Controls
{
    /// <summary>
    /// Codebehind for movie control
    /// </summary>
    internal partial class MovieControl : UserControl
    {
        #region Constructor
        internal MovieControl()
        {
            InitializeComponent();
            Loaded += (sender, eventArgs) =>
                {
                    LoadMovies();
                };
        }
        #endregion

        #region Fields
        private ObservableCollection<DVD> Movies
        {
            set;
            get;
        }
        #endregion

        #region Methods
        private void ButtonAdd_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                var movie = InitializeMovie();

                if (!Movies.Contains(movie))
                    Movies.Add(movie);

                RentalService.Current.GetRepository<DVD>().Add(movie);
            }
            catch(Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ButtonUpdate_Click(object sender, RoutedEventArgs e)
        {
            var movie = ListViewMovies.SelectedItem as DVD;

            if (movie == null)
            {
                MessageBox.Show("Wählen Sie zuerst einen Film aus");
                return; 
            }

            var currentMovie = Movies.FirstOrDefault(aMovie => aMovie.ID.Equals(movie.ID));

            if (currentMovie == null)
            {
                MessageBox.Show("Zurzeit existiert kein entsprechender Eintrag");
                return; 
            }

            try
            {
                currentMovie.Title = TextBoxTitle.Text;
                currentMovie.Price = decimal.Parse(TextBoxPrice.Text);
                currentMovie.AgeRating = int.Parse(TextBoxAgeRating.Text);

                RentalService.Current.GetRepository<DVD>().Update(currentMovie);
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ButtonDelete_Click(object sender, RoutedEventArgs e)
        {
            var movie = ListViewMovies.SelectedItem as DVD;
            
            if (movie == null)
            {
                MessageBox.Show("Wählen Sie zuerst einen Film aus");
                return; 
            }

            try
            {
                RentalService.Current.GetRepository<DVD>().Delete(movie.ID);
                Movies.Remove(movie);
                DataContext = Movies;
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void CheckBoxRentedUnChecked(object sender, RoutedEventArgs e)
        {
            DataContext = Movies.Where(movie => movie.Rental == null);
        }

        private void CheckBoxRentedChecked(object sender, RoutedEventArgs e)
        {
            DataContext = Movies.Where(movie => movie.Rental != null);
        }

        private void TextBoxTitle_TextChanged(object sender, TextChangedEventArgs e)
        {
            var changedText = (e.Source as TextBox).Text;

            if (string.IsNullOrEmpty(changedText))
                DataContext = Movies;
            else 
             DataContext = Movies.Where(movie => movie.Title.Equals(changedText));
        }

        private void TextBoxAgeRating_TextChanged(object sender, TextChangedEventArgs e)
        {
            int currentAgeRating;

            if (int.TryParse((e.Source as TextBox).Text, out currentAgeRating))
                DataContext = Movies.Where(movie => movie.AgeRating == currentAgeRating);
            else
                DataContext = Movies;
        }

        private void LoadMovies()
        {
            try
            {
                Movies = new ObservableCollection<DVD>(RentalService.Current.GetRepository<DVD>().GetAll());
                DataContext = Movies;
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private DVD InitializeMovie()
        {
            var movie = new DVD();
            movie.Title = TextBoxTitle.Text;
            movie.Price = decimal.Parse(TextBoxPrice.Text);
            movie.AgeRating = int.Parse(TextBoxAgeRating.Text);

            return movie;
        }
        #endregion
    }
}
