﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;

namespace Exercise_12
{
    /// <summary>
    /// Interaktionslogik für BookView.xaml
    /// </summary>
    public partial class BookView : UserControl
    {
        public BookView()
        {
            InitializeComponent();
            
            LoadCustomers();
            LoadBooks();

            DataContext = Books;
        }

        private void LoadBooks()
        {
            using (var context = new BookStoreEntities())
            {
                if (ComboBoxCustomerFilter.SelectedItem != null)
                {
                    var selectedCustomer = ComboBoxCustomerFilter.SelectedItem as Customer;

                    Books = new ObservableCollection<Book>(context.Customer.FirstOrDefault(aCustomer => aCustomer.ID == selectedCustomer.ID).Book.ToList());
                }
            }
        }

        private void LoadCustomers()
        {
            using (var context = new BookStoreEntities())
            {
                ComboBoxCustomerFilter.ItemsSource = context.Customer.ToList();
            }
        }

        private void MenuItemDelete_Click(object sender, RoutedEventArgs e)
        {
            if (ListViewBooks.SelectedItem != null)
            {
                var selectedBook = ListViewBooks.SelectedItem as Book;

                Books.Remove(selectedBook);

                using (var context = new BookStoreEntities())
                {
                    var dbBook = context.Book.FirstOrDefault(aBook => aBook.ID == selectedBook.ID);
                    context.Book.DeleteObject(dbBook);
                    context.SaveChanges();
                }
            }
        }

        private void ButtonAdd_Click(object sender, RoutedEventArgs e)
        {
            var currentBook = new Book { Title = TextBoxTitle.Text, Description = TextBoxDescription.Text, Price = decimal.Parse(TextBoxPrice.Text) };
            var selectedCustomer = ComboBoxCustomerFilter.SelectedItem as Customer;

            if (selectedCustomer != null)
            {
                using (var context = new BookStoreEntities())
                {
                    context.Customer.FirstOrDefault(aCustomer => aCustomer.ID == selectedCustomer.ID).Book.Add(currentBook);
                    context.SaveChanges();

                    Books.Add(currentBook);
                }
            }

        }

        private void ButtonUpdate_Click(object sender, RoutedEventArgs e)
        {
            if (ListViewBooks.SelectedItem != null)
            {
                var selectedBook = ListViewBooks.SelectedItem as Book;

                using (var context = new BookStoreEntities())
                {
                    var dbBook = context.Book.FirstOrDefault(aBook => aBook.ID == selectedBook.ID);
                    dbBook.Title = TextBoxTitle.Text;
                    dbBook.Description = TextBoxDescription.Text;
                    dbBook.Price = decimal.Parse(TextBoxPrice.Text);

                    context.SaveChanges();

                    Books.FirstOrDefault(aBook => aBook.ID == selectedBook.ID).Title = TextBoxTitle.Text;
                    Books.FirstOrDefault(aBook => aBook.ID == selectedBook.ID).Description = TextBoxDescription.Text;
                    Books.FirstOrDefault(aBook => aBook.ID == selectedBook.ID).Price = decimal.Parse(TextBoxPrice.Text);
                }
            }
        }

        public ObservableCollection<Book> Books
        {
            private set;
            get;
        }

        private void ComboBoxCustomerFilter_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            LoadBooks();
        }
    }
}
