﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace PersonnelManagement
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private PersonnelManagement _personnelManagement;
        public MainWindow()
        {
            InitializeComponent();
            _personnelManagement = new PersonnelManagement();
           DataContext =  _personnelManagement.LoadAllPersons();
        }


        private void ButtonAdd_Click(object sender, RoutedEventArgs e)
        {
            _personnelManagement.SavePerson(GetPerson());
            DataContext = _personnelManagement.LoadAllPersons();
        }

        private void ButtonUpdate_Click(object sender, RoutedEventArgs e)
        {
            if (ListViewPersons.SelectedItem == null)
                return;

            if (!_personnelManagement.UpdatePerson(GetPerson()))
            {
                MessageBox.Show("Fehler beim Aktualisieren von Daten");
                return;
            }

            DataContext = _personnelManagement.LoadAllPersons();
        }

        private void ButtonDelete_Click(object sender, RoutedEventArgs e)
        {
            if (ListViewPersons.SelectedItem == null)
                return;

            if(!_personnelManagement.DeletePerson(GetPerson()))
            {
                MessageBox.Show("Fehler beim Löschen von Daten");
                return;
            }

            DataContext = _personnelManagement.LoadAllPersons();
        }

        private void ButtonCalculate_Click(object sender, RoutedEventArgs e)
        {
            TextBoxMonthlyRate.Text = _personnelManagement.CalculateSalary(GetPerson()).ToString();
        }

        private void ButtonPromote_Click(object sender, RoutedEventArgs e)
        {
            TextBoxPromoteState.Text = _personnelManagement.Promote(GetPerson() as IPromotable);
        }

        private Person GetPerson()
        {
            Person person;

            if (RadioButtonPermantEmployee.IsChecked == true)
            {
                person = new PermanentEmployee(TextBoxFirstName.Text, TextBoxLastName.Text, TextBoxAddress.Text,
                                               int.Parse(TextBoxAge.Text), decimal.Parse(TextBoxHourlyRate.Text));
                person.EmployeeType = "PermanentEmployee";
            }
            else
            {
                person = new Freelancer(TextBoxFirstName.Text, TextBoxLastName.Text, TextBoxAddress.Text,
                                        int.Parse(TextBoxAge.Text), decimal.Parse(TextBoxHourlyRate.Text));
                person.EmployeeType = "Freelancer";
            }

            if (ListViewPersons.SelectedItem != null)
                person.ID = (ListViewPersons.SelectedItem as Person).ID;
            return person;
        }
    }
}
