﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

namespace PersonnelManagement
{
    public class PersonnelManagement
    {
        public decimal CalculateSalary(Person person)
        {
            return person.CalculateSalary();
        }

        public string Promote(IPromotable promotable)
        {
            return promotable.Promote();
        }

        public void SavePerson(Person person)
        {
            Person conformPerson = GetDatabaseConformPerson(person);

            using(PersonnelManagementEntities context = new PersonnelManagementEntities())
            {
                context.AddToPerson(conformPerson);
                context.SaveChanges();
            }
        }

        public bool UpdatePerson(Person person)
        {
            using (PersonnelManagementEntities context = new PersonnelManagementEntities())
            {
                var foundPerson = context.Person.FirstOrDefault(candidate => candidate.ID == person.ID);
                if (foundPerson == null)
                    return false;

                foundPerson.FirstName = person.FirstName;
                foundPerson.LastName = person.LastName;
                foundPerson.Age = person.Age;
                foundPerson.Address = person.Address;
                foundPerson.HourlyRate = person.HourlyRate;
                foundPerson.EmployeeType = person.EmployeeType;

                context.SaveChanges();
                return true;
            }
        }

        public bool DeletePerson(Person person)
        {
            using (PersonnelManagementEntities context = new PersonnelManagementEntities())
            {
                var foundPerson = context.Person.FirstOrDefault(candidate => candidate.ID == person.ID);
                if (foundPerson == null)
                    return false;

               context.Person.DeleteObject(foundPerson);

                context.SaveChanges();
                return true;
            }
        }

        private static Person GetDatabaseConformPerson(Person person)
        {
            var temporaryPerson = new Person()
                                      {
                                          Address = person.Address,
                                          Age = person.Age,
                                          FirstName = person.FirstName,
                                          HourlyRate = person.HourlyRate,
                                          LastName = person.LastName,
                                          ID = person.ID
                                      };

            if (person is Freelancer)
            {
                temporaryPerson.EmployeeType = "Freelancer";
            }
            else
                temporaryPerson.EmployeeType = "PermanentEmployee";
            return temporaryPerson;
        }

        public ObservableCollection<Person> LoadAllPersons()
        {
            using (var context = new PersonnelManagementEntities())
            {
                return new ObservableCollection<Person>(context.Person);
            }
        }
    }
}
