﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using CMA.MovieRentalStore.EnterpriseModel;
using CMA.MovieRentalStore.EnterpriseModel.Logging;

namespace CMA.MovieRentalStore.Presentation.Controls
{
    /// <summary>
    /// Codebehind für Person control
    /// </summary>
    internal partial class PersonControl : UserControl
    {
        #region Constructor
        internal PersonControl()
        {
            InitializeComponent();

            Loaded += (sender, eventArgs) =>
            {
                LoadPersons();
            };
        }
        #endregion

        #region Fields
        private ObservableCollection<Person> Persons
        {
            set;
            get;
        }
        #endregion

        #region Methods
        private void ButtonAdd_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                var person = InitializePerson();

                if (!Persons.Contains(person))
                    Persons.Add(person);

                RentalService.Current.GetRepository<Person>().Add(person);
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ButtonUpdate_Click(object sender, RoutedEventArgs e)
        {
            var person = ListViewPersons.SelectedItem as Person;

            if (person == null)
            {
                MessageBox.Show("Wählen Sie zuerst eine Person aus");
                return; 
            }

            var currentPerson = Persons.FirstOrDefault(aPerson => aPerson.ID.Equals(person.ID));

            if (currentPerson == null)
            {
                MessageBox.Show("Ein entsprechender Eintrag existiert nicht");
                return; 
            }

            try
            {
                currentPerson.FirstName = TextBoxFirstName.Text;
                currentPerson.LastName = TextBoxLastName.Text;
                currentPerson.Age = int.Parse(TextBoxAge.Text);
                currentPerson.Address = TextBoxAddress.Text;

                RentalService.Current.GetRepository<Person>().Update(currentPerson);
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void ButtonDelete_Click(object sender, RoutedEventArgs e)
        {
            var person = ListViewPersons.SelectedItem as Person;
           
            if (person == null)
            {
                MessageBox.Show("Wählen Sie zuerst eine Person aus");
                return; 
            }

            try
            {
                RentalService.Current.GetRepository<Person>().Delete(person.ID);
                Persons.Remove(person);
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private void LoadPersons()
        {
            try
            {
                Persons = new ObservableCollection<Person>(RentalService.Current.GetRepository<Person>().GetAll());
                DataContext = Persons;
            }
            catch (Exception exception)
            {
                ExceptionLogger.LogException(exception);
                MessageBox.Show("Ein Fehler ist aufgetreten");
            }
        }

        private Person InitializePerson()
        {
            var person = new Person();

            person.FirstName = TextBoxFirstName.Text;
            person.LastName = TextBoxLastName.Text;
            person.Age = int.Parse(TextBoxAge.Text);
            person.Address = TextBoxAddress.Text;

            return person;
        }
        #endregion
    }
}
