﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace SimpleCalculator
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Calculator _calculator;

        public MainWindow()
        {
            InitializeComponent();
            _calculator = new Calculator();
        }

        private void MenuReset_Click(object sender, RoutedEventArgs e)
        {
            //Reset all controls and their values
            txtBxFirstValue.Text = txtBxSecondValue.Text = txtBxResult.Text = string.Empty;
            cmbBxSymbols.SelectedIndex = 0;
        }

        private void MenuExit_Click(object sender, RoutedEventArgs e)
        {
            //Close application
            Close();
        }

        private void ContextMenuItem_Click(object sender, RoutedEventArgs e)
        {
            //Get clicked context menu entry
            var currentContextMenuItem = sender as MenuItem;

            if (currentContextMenuItem == null)
                return;

            //Depending on selected context menu entry, entry for combobox with calculation symbols 
            //will be selected
            switch (currentContextMenuItem.Header.ToString())
            {
                case "Addition":
                    cmbBxSymbols.SelectedIndex = 0;
                    break;
                    case "Subtraktion":
                    cmbBxSymbols.SelectedIndex = 1;
                    break;
                case "Multiplikation":
                    cmbBxSymbols.SelectedIndex = 2;
                    break;
                case "Division":
                    cmbBxSymbols.SelectedIndex = 3;
                    break;
            }
        }

        private void ContextMenuResult_Click(object sender, RoutedEventArgs e)
        {
            //Call eventhandler for button 'equals'
            ButtonEquals_Click(btnEquals, e);
        }

        private void ButtonNumber_Click(object sender, RoutedEventArgs e)
        {
            var pressedButtonNumber = sender as Button;

            if(pressedButtonNumber == null)
                return;

            //Get focused control of window
            var focusedControl = FocusManager.GetFocusedElement(this);

            //Set textbox content depending on pressed number button
            if (focusedControl is TextBox)
                (focusedControl as TextBox).Text = pressedButtonNumber.Content.ToString();
        }

        private void ButtonEquals_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                //Get both values
                int firstValue = int.Parse(txtBxFirstValue.Text);
                int secondValue = int.Parse(txtBxSecondValue.Text);

                //Depending on selected symbol, call specific calculator method and return result
                switch (cmbBxSymbols.SelectedIndex)
                {
                    case 0:
                        txtBxResult.Text = _calculator.Calculate(new Addition(),firstValue, secondValue).ToString();
                        break;
                    case 1:
                        txtBxResult.Text = _calculator.Calculate(new Subtraction(),firstValue, secondValue).ToString();
                        break;
                    case 2:
                        txtBxResult.Text = _calculator.Calculate(new Multiplication(),firstValue, secondValue).ToString();
                        break;
                    case 3:
                        txtBxResult.Text = _calculator.Calculate(new Division(), firstValue, secondValue).ToString();
                        break;
                    default:
                        MessageBox.Show("Kein Rechensymbol ausgewählt");
                        break;
                }
            }
            catch
            {
                MessageBox.Show("Der erste oder der zweite Wert ist keine Ganzzahl");
            }
        }
    }
}
