﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Exercise_6
{
    public delegate Fraction<T> CalculationDefinition<T>(Fraction<T> first, Fraction<T> second) where T : IComparable;

    public class Fraction<T> where T : IComparable
    {
        public event Action<Fraction<T>> CalculationFinished;

        public Fraction(T numerator, T denominator)
        {
            Numerator = numerator;
            Denominator = denominator;
        }

        public T Numerator
        {
            private set;
            get;
        }

        public T Denominator
        {
            private set;
            get;
        }

        public override string ToString()
        {
            return string.Format("{0}/{1}", Numerator, Denominator);
        }

        //Override equals
        public override bool Equals(object obj)
        {
            //False, if not of same type
            if(!(obj is Fraction<T>))
                return false;

            var otherFraction = obj as Fraction<T>;

            return otherFraction.Numerator.Equals(Numerator) && otherFraction.Denominator.Equals(Denominator);
        }

        public void Calculate(CalculationDefinition<T> calculationDefinition, Fraction<T> fraction)
        {
            var calculatedFraction = calculationDefinition(this, fraction);

            if (CalculationFinished != null)
                CalculationFinished(calculatedFraction);
        }
    }
}
