﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;


namespace PersonnelManagement
{
  public partial class MainForm : Form
  {
    private PersonnelManagement _personnelManagement;

    public MainForm()
    {
      InitializeComponent();
      _personnelManagement = new PersonnelManagement();
    }


    private Person GetInitializedPerson()
    {
      string firstName = txtBxFirstName.Text;
      string lastName = txtBxLastName.Text;
      string address = txtBxAddress.Text;
      uint age = uint.Parse(txtBxAge.Text);
      decimal hourlyRate = decimal.Parse(txtBxHourlyRate.Text);

      if (rdbtnPermanentEmployee.Checked)
        return new PermanentEmployee(firstName, lastName, address, age, hourlyRate);
      else
        return new Freelancer(firstName, lastName, address, age, hourlyRate);
    }


    private void btnCalculateSalary_Click(object sender, EventArgs e)
    {
      Person person = GetInitializedPerson();
      txtBxTotalSalary.Text = _personnelManagement.CalculateSalary(person).ToString();
    }


    private void btnPromote_Click(object sender, EventArgs e)
    {
      Person person = GetInitializedPerson();
      IPromotable promotable = null;

      if (person is Freelancer)
        promotable = person as Freelancer;
      else if (person is PermanentEmployee)
        promotable = person as PermanentEmployee;

      txtBxResultPromotion.Text = _personnelManagement.Promote(promotable);
    }


    private void btnAdd_Click(object sender, EventArgs e)
    {
      Person person = GetInitializedPerson();
      if (_personnelManagement.Add(person))
        lstbxEmployees.Items.Add(person);
    }


    private void btnRemove_Click(object sender, EventArgs e)
    {
      var selectedPerson = lstbxEmployees.SelectedItem as Person;
      if (selectedPerson == null)
      {
        MessageBox.Show("Bitte einen Mitarbeiter aus der Liste auswählen.");
        return;
      }

      if (_personnelManagement.Remove(selectedPerson))
      {
        lstbxEmployees.Items.Remove(selectedPerson);
        ResetControlsToDefault();
      }
    }


    private void ResetControlsToDefault()
    {
      txtBxFirstName.Text =
        txtBxLastName.Text =
        txtBxAddress.Text = txtBxAge.Text = txtBxHourlyRate.Text = txtBxResultPromotion.Text = txtBxTotalSalary.Text = string.Empty;
      rdbtnPermanentEmployee.Checked = true;
      rdbtnFreelancer.Checked = false;
    }


    private void lstbxEmployees_SelectedIndexChanged(object sender, EventArgs e)
    {
      var selectedPerson = lstbxEmployees.SelectedItem as Person;
      if (selectedPerson == null)
      {
        btnRemove.Enabled = false;
        return;
      }

      btnRemove.Enabled = true;
    }
  }
}