﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace PersonnelManagement
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        PersonnelManagement _personnelManagement;

        public MainWindow()
        {
            InitializeComponent();
            _personnelManagement = new PersonnelManagement();
        }

        private void btnCalculateSalary_Click(object sender, RoutedEventArgs e)
        {
            Person person = GetInitializedPerson();
            txtBxTotalSalary.Text =
            _personnelManagement.CalculateSalary(person).ToString();
        }

        private void btnPromote_Click(object sender, RoutedEventArgs e)
        {
            Person person = GetInitializedPerson();
            IPromotable promotable = null;

            if (person is IPromotable)
            {
                promotable = person as IPromotable;
                txtBxPromotion.Text =
                _personnelManagement.Promote(promotable);
            }
            else
                txtBxPromotion.Text = "Kann nicht befördert werden.";
        }

        private Person GetInitializedPerson()
        {
            string firstName = txtBxFirstName.Text;
            string lastName = txtBxLastName.Text;
            string address = txtBxAddress.Text;
            uint age = uint.Parse(txtBxAge.Text);
            decimal hourlyRate = decimal.Parse(txtBxHourlyRate.Text);

            if (rdbtnPermanentEmployee.IsChecked == true) //Überprüfung der 
                //Checked Eigenschaft
                return new PermanentEmployee(firstName, lastName, address, age,
                hourlyRate); //Festangestellter ausgewählt
            else
                return new Freelancer(firstName, lastName, address,
                                      age, hourlyRate);
        }

        private void lstBxEmployees_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var selectedPerson = lstBxEmployees.SelectedItem as Person;
            if (selectedPerson == null)
            {
                btnRemove.IsEnabled = false;
                return;
            }
            btnRemove.IsEnabled = true;

        }

        //Eventhandler des Click-Ereignis für den Button
        //Hinzufügen
        private void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            Person person = GetInitializedPerson();
            if (_personnelManagement.Add(person))
            {
                //Neues Person-Objekt der Liste hinzufügen
                lstBxEmployees.Items.Add(person);
            }
        }

        //Eventhandler des Click-Ereignis für den Button
        //Löschen
        private void btnRemove_Click(object sender, RoutedEventArgs e)
        {
            //Selektierte Person aus der Liste holen
            var selectedPerson = lstBxEmployees.SelectedItem
            as Person;
            if (selectedPerson == null)
            {
                MessageBox.Show("Bitte einen Mitarbeiter aus " +
                "der Liste auswählen");
                return;
            }

            if (_personnelManagement.Remove(selectedPerson))
            {
                lstBxEmployees.Items.Remove(selectedPerson);
                ResetControlsToDefault();
            }
        }

        //Alle TextBox-Inhalte leeren
        private void ResetControlsToDefault()
        {
            txtBxFirstName.Text =
            txtBxLastName.Text =
            txtBxAddress.Text = txtBxAge.Text =
            txtBxHourlyRate.Text =
            txtBxPromotion.Text =
            txtBxTotalSalary.Text = string.Empty;
            rdbtnPermanentEmployee.IsChecked = true;
            rdbtnFreelancer.IsChecked = false;
        }

    }
}
