﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace PersonnelManagement
{
    public class PersonnelManagement : IEnumerable<Person>
    {
        //Dynamische Speicherung mehrerer Personen
        private List<Person> _persons;

        public PersonnelManagement()
        {
            _persons = new List<Person>();
        }

        public decimal CalculateSalary(Person person)
        {
            return person.CalculateSalary();
        }

        public string Promote(IPromotable promotable)
        {
            return promotable.Promote();
        }


        public bool Add(Person person)
        {
            if (!_persons.Contains(person))
            {
                _persons.Add(person);
                return true;
            }
            return false;
        }

        public bool Remove(Person person)
        {
            return _persons.Remove(person);
        }

        public IEnumerator<Person> GetEnumerator()
        {
            return new PersonEnumerator(_persons);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return new PersonEnumerator(_persons);
        }

        private class PersonEnumerator : IEnumerator<Person>
        {
            private readonly List<Person> _persons;
            private int _position = -1;

            internal PersonEnumerator(List<Person> persons)
            {
                _persons = persons;
            }

            public Person Current
            {
                get
                {
                    if (_position < _persons.Count)
                        return _persons[_position]; //Aktuelles Element
                    return null; //Ende der Liste
                }
            }

            public void Dispose()
            {
                //TODO: Add disposable parts here
            }

            object System.Collections.IEnumerator.Current
            {
                get
                {
                    //Spezifische Variante für nicht generisches Interface 
                    //IEnumerable
                    if (_position < _persons.Count)
                        return _persons[_position];
                    return null;
                }
            }

            public bool MoveNext()
            {
                _position++; //Zeiger um eine Position verschieben
                return (_position < _persons.Count); //Prüfen, ob
                //Ende der Liste noch nicht erreicht
            }

            public void Reset()
            {
                _position = -1; //Zurücksetzen des Zeigers auf den Anfang
            }
        }
    }
}
